#ifndef xpsock_h__
#define xpsock_h__
/*
 * xp_sock.h - socket stuff
 * Copyright (C) 1998, 1999, 2000 Rex Feany <laeos@laeos.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * %W%
 *
 */

#include "dbuf.h"


/* Flags a xpsock can have */
#define SK_DEAD		0x0001	/* closed, dead, gone. */
#define SK_ACTIVE	0x0002	/* in use already. don't touch me. */
#define SK_BACKEDUP	0x0004	/* write would block; queue the request */
#define SK_LISTEN	0x0008	/* this is a listening socket */

/* what we called this event callback function for */
enum xpsock_event
{
	xpsock_event_close,	/* we are closing the socket. */
	xpsock_event_data,	/* data is ready to be read   */
	xpsock_event_timeout, 	/* timeout was reached        */
	xpsock_event_error,	/* check xp_error(sk)	      */
	xpsock_event_accept	/* new connection             */
};


/* types we use around here */

typedef struct xpsock_ xpsock;
typedef int (*xp_iface_cb)(void *data, struct xpsock_iflist **list);
typedef int (*xp_ev_cb)(void *data, xpsock *xp, enum xpsock_event);
typedef void (*xp_lookup_cb)(void *data, struct addrinfo *ai);


/* one of these exists for every open socket */
struct xpsock_ {

	/* free form name anyone can use */	
	const char *sk_name;

	int sk_fd;
	int sk_timeout;
	u_int32_t sk_flags;

	/* This is what does everything! */
	xp_ev_cb sk_callback;
	void *sk_data;

	/* here for convience, possibly NULL */
	char *sk_peer_name;
	struct sockaddr	*sk_peer;
	size_t sk_peer_len;

	char *sk_local_name;
	struct sockaddr *sk_local;
	size_t sk_local_len;

	/* mmm statistics */
	u_int32_t sk_bytes_in;
	u_int32_t sk_kbytes_in;

	u_int32_t sk_bytes_out;
	u_int32_t sk_kbytes_out;

	/* what was the last error? */
	const char *sk_err;
	int sk_errno;

	dbuf sk_rbuf;
	dbuf sk_wbuf;
};


/* Get stored data from the socket structure */
#define xp_data(type, sk)	((type *)(sk)->sk_data)

/* Reset the timeout */
#define xp_rstimeout(sk)	((sk)->sk_timeout = time(NULL))

/* Set the name of this socket */
#define xp_setname(sk, name)		((sk)->sk_name = (name))

/* get the name */
#define xp_getname(sk)			((sk)->sk_name)

/* can we play with this socket? */
#define xp_touchable(sk)		(!((sk)->sk_flags & (SK_ACTIVE|SK_DEAD)))

/* is there data already queued for this socket? */
#define xp_backedup(sk)			((sk)->sk_flags & (SK_BACKEDUP));

/* string describing last error */
#define xp_error(sk)			((sk)->sk_err)

/* what the errno value was */
#define xp_errno(sk)			((sk)->sk_errno)

/* mmm statistics */
#define xp_add_bytes(bytes, kilos, addme) 	\
	(bytes) += (addme); 			\
	(kilos) += ((bytes)&0x3ff)>>10; 	\
	(bytes) &= 0x3ff; 

#define xp_sent(sk, len) 	xp_add_bytes( (sk)->sk_bytes_out, (sk)->sk_kbytes_out, (len))
#define xp_recv(sk, len)	xp_add_bytes( (sk)->sk_bytes_in, (sk)->sk_kbytes_in, (len))

/* calls callback with a list of interfaces (see xp_ifinfo.h) and data */
int xp_ifaces(xp_iface_cb callback, void *data);

/* set the default hostname to host */
int xp_setiface(const char *host);

/* Get the default hostname */
const char *host xp_getiface(void);

/* connect to host/port. callback is used for notification of activity */
xpsock * xp_connect(const char *host, short port, xp_ev_cb callback, void *data);

/* passive open, callback is passed newly created socket (from accept) */
xpsock * xp_listen(short port, xp_ev_cb callback, void *data);

/* async hostname lookup */
int xp_lookup(char *host, xp_lookup_cb callback, void *data);

/* blocking hostname lookup */
int xp_lookup_now(char *host, struct addrinfo **ai);

/* do a poll/select with this many seconds timeout */
void xp_run(int sec);

/* close / deallocate this socket */
void xp_close(xpsock *sk);

/* Read a line of data from this socket. len is maxlen.*/
/* returns: -1 failure, 0 - len bytes read, len+1 no end of line, truncated */
int xp_readline(xpsock *sk, void *buf, size_t len);

/* Read this much data to a socket, returns just like read(2) */
int xp_read(xpsock *sk, void *buf, size_t len);

/* Write this much data to a socket returns -1 error, 0 ok */
int xp_write(xpsock *sk, void *buf, size_t len);

/* specialized. write list of strings, seperated by spaces */
int xp_writes(xpsock *sk, char *, ...);

#endif /* xpsock_h__ */

